@@ -85,23 +85,31 @@ namespace persistent_data {
 		// different sub tree (by looking at the btree_path).
 		class path_tracker {
 		public:
+			path_tracker() {
+				// We push an empty path, to ensure there
+				// is always a current_path.
+				paths_.push_back(btree_path());
+			}
+
 			// returns the old path if the tree has changed.
-			boost::optional<btree_path> next_path(btree_path const &p) {
-				if (p != path_) {
-					btree_path tmp(path_);
-					path_ = p;
-					return boost::optional<btree_path>(tmp);
+			btree_path const *next_path(btree_path const &p) {
+				if (p != current_path()) {
+					if (paths_.size() == 2)
+						paths_.pop_front();
+					paths_.push_back(p);
+
+					return &paths_.front();
 				}
 
-				return boost::optional<btree_path>();
+				return NULL;
 			}
 
 			btree_path const &current_path() const {
-				return path_;
+				return paths_.back();
 			}
 
 		private:
-			btree_path path_;
+			std::list<btree_path> paths_;
 		};
 
 		//----------------------------------------------------------------
@@ -136,11 +144,9 @@ namespace persistent_data {
 			typedef run<block_address> run64;
 			typedef boost::optional<run64> maybe_run64;
 
-			btree_damage_visitor(block_counter &counter,
-					     ValueVisitor &value_visitor,
+			btree_damage_visitor(ValueVisitor &value_visitor,
 					     DamageVisitor &damage_visitor)
-				: counter_(counter),
-				  avoid_repeated_visits_(true),
+				: avoid_repeated_visits_(true),
 				  value_visitor_(value_visitor),
 				  damage_visitor_(damage_visitor) {
 			}
@@ -191,11 +197,12 @@ namespace persistent_data {
 		private:
 			void visit_values(btree_path const &path,
 					  node_ref<ValueTraits> const &n) {
+				btree_path p2(path);
 				unsigned nr = n.get_nr_entries();
 				for (unsigned i = 0; i < nr; i++) {
-					btree_path p2(path);
 					p2.push_back(n.key_at(i));
 					value_visitor_.visit(p2, n.value_at(i));
+					p2.pop_back();
 				}
 			}
 
@@ -243,8 +250,6 @@ namespace persistent_data {
 			bool already_visited(node const &n) {
 				block_address b = n.get_location();
 
-				counter_.inc(b);
-
 				if (avoid_repeated_visits_) {
 					if (seen_.count(b) > 0)
 						return true;
@@ -431,7 +436,7 @@ namespace persistent_data {
 			}
 
 			void update_path(btree_path const &path) {
-				boost::optional<btree_path> old_path = path_tracker_.next_path(path);
+				btree_path const *old_path = path_tracker_.next_path(path);
 				if (old_path)
 					// we need to emit any errors that
 					// were accrued against the old
@@ -441,7 +446,6 @@ namespace persistent_data {
 
 			//--------------------------------
 
-			block_counter &counter_;
 			bool avoid_repeated_visits_;
 
 			ValueVisitor &value_visitor_;
@@ -458,11 +462,10 @@ namespace persistent_data {
 
 	template <unsigned Levels, typename ValueTraits, typename ValueVisitor, typename DamageVisitor>
 	void btree_visit_values(btree<Levels, ValueTraits> const &tree,
-				block_counter &counter,
 				ValueVisitor &value_visitor,
 				DamageVisitor &damage_visitor) {
 		btree_detail::btree_damage_visitor<ValueVisitor, DamageVisitor, Levels, ValueTraits>
-			v(counter, value_visitor, damage_visitor);
+			v(value_visitor, damage_visitor);
 		tree.visit_depth_first(v);
 	}
 }
