@@ -10,7 +10,7 @@
 // FIXME: move to a different unit
 namespace persistent_data {
 	persistent_data::block_address get_nr_blocks(string const &path);
-	block_manager<>::ptr open_bm(std::string const &dev_path, block_io<>::mode m);
+	block_manager<>::ptr open_bm(std::string const &dev_path, block_manager<>::mode m);
 
 	void check_file_exists(std::string const &file);
 }
